import os
import csv
import pandas as pd
import asyncio
from playwright.async_api import async_playwright
from bs4 import BeautifulSoup

OUTPUT_FILE = 'minfin_books.csv'
START_URL = 'https://minfin.gov.ru/ru/ministry/historylib/'
WAIT_SECONDS = 2  # задержка в секундах между скроллингами

def load_existing_links():
    """Загрузка уже собранных source_link из файла CSV"""
    if not os.path.exists(OUTPUT_FILE):
        return set()
    df = pd.read_csv(OUTPUT_FILE)
    return set(df['source_link'].values)

def append_to_csv(book_data):
    """Добавление новых записей в CSV"""
    file_exists = os.path.exists(OUTPUT_FILE)
    with open(OUTPUT_FILE, 'a', newline='', encoding='utf-8') as f:
        fieldnames = ['name', 'source_link', 'book_link']
        writer = csv.DictWriter(f, fieldnames=fieldnames)
        if not file_exists:
            writer.writeheader()
        for row in book_data:
            writer.writerow(row)

async def scroll_to_bottom(page):
    """Плавный скролл страницы вниз для подгрузки всех плиток"""
    previous_height = await page.evaluate("document.body.scrollHeight")
    while True:
        await page.evaluate("window.scrollTo(0, document.body.scrollHeight)")
        await asyncio.sleep(WAIT_SECONDS)
        height = await page.evaluate("document.body.scrollHeight")
        if height == previous_height:
            break
        previous_height = height

def extract_books_from_html(html, existing_links):
    """Парсинг HTML и извлечение карточек книг"""
    soup = BeautifulSoup(html, "html.parser")
    tiles = soup.select(".history-tile")
    books = []
    for tile in tiles:
        try:
            title_tag = tile.select_one(".document-title a")
            if not title_tag:
                continue
            name = title_tag.text.strip()
            source_link = "https://minfin.gov.ru" + title_tag['href']
            if source_link in existing_links:
                continue
            download_btn = tile.select_one('a[title*="Скачать"]')
            if not download_btn:
                continue
            pdf_link = 'https://minfin.gov.ru' + download_btn['href']
            books.append({
                'name': name,
                'source_link': source_link,
                'book_link': pdf_link
            })
        except Exception as e:
            print(f"Ошибка разбора плитки: {e}")
            continue
    return books

async def main():
    existing_links = load_existing_links()
    print(f"📄 Уже записей в файле: {len(existing_links)}")

    async with async_playwright() as p:
        browser = await p.chromium.launch(headless=True)
        context = await browser.new_context(user_agent=(
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
            "AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/125.0.0.0 Safari/537.36"
        ))
        page = await context.new_page()
        await page.goto(START_URL, timeout=60_000)
        print("🌍 Загрузили страницу, начинаем скроллинг...")
        await scroll_to_bottom(page)
        print("✅ Динамическая подгрузка завершилась")

        html = await page.content()
        books = extract_books_from_html(html, existing_links)

        print(f"📚 Найдено новых книг: {len(books)}")

        if books:
            append_to_csv(books)
            print(f"✅ Добавлено в файл: {len(books)} записей")
        else:
            print("🔍 Новых записей не найдено")

        await browser.close()

if __name__ == "__main__":
    try:
        asyncio.run(main())
    except Exception as e:
        print(f"💥 Ошибка выполнения скрипта: {e}")
